def _setup_ctypes ():
    import os 
    from ctypes .macholib import dyld 

    frameworks =os .path .join (os .environ ["RESOURCEPATH"],"..","Frameworks")
    dyld .DEFAULT_FRAMEWORK_FALLBACK .insert (0 ,frameworks )
    dyld .DEFAULT_LIBRARY_FALLBACK .insert (0 ,frameworks )


_setup_ctypes ()
